package cmdVM

import (
    "nexus-common/params"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type ExportDir struct {
    Name string
}

func (cmd *ExportDir)GetName() string {
    return cmd.Name
}

func (cmd *ExportDir)GetDesc() []string {
    return []string{
        "Exports one or more VMs' directory into one or more compressed archives.",
        "If not the VM's owner: requires VM_READFS VM access capability or VM_READFS_ANY user capability."}
}

func (cmd *ExportDir)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: ",cmd.GetName()," [ID ...] [regex ...] vmDir")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    const usage string = `vmDir    Directory (in the VM) to export into a compressed archive.
Remark: one archive, named after the VM's name, is created per VM.`
    u.PrintlnErr(usage)
    u.PrintlnErr("")
    printRegexUsageDetails()
}

func (cmd *ExportDir)Run(args []string) int {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    argc := len(args)
    if argc < 2 {
        cmd.PrintUsage()
        return 1
    }

    dir := args[argc-1]

    vms, err := getFilteredVMs("/vms/exportdir", args[:argc-1])
    if err != nil {
        u.PrintlnErr("Error: "+err.Error())
        return 1
    }

    if len(vms) == 0 {
        u.PrintlnErr("Error: VM(s) not found.")
        return 1
    }

    params := &params.VMExportDir { Dir: dir }

    statusCode := 0

    client.SetAllowGetMethodPayload(true)

    for _, vm := range(vms) {
        uuid := vm.ID.String()
        outputFile := vm.Name+"_"+vm.ID.String()+".tar.gz"
        resp, err := client.R().SetOutput(outputFile).SetBody(params).Get(host+"/vms/"+uuid+"/exportdir")
        if err != nil {
            u.PrintlnErr("Failed exporting "+dir+" from VM \""+vm.Name+"\": "+err.Error())
            statusCode = 1
        } else {
            if resp.IsSuccess() {
                u.Println("Exported "+dir+" from VM \""+vm.Name+"\" into "+outputFile)
            } else {
                errorMsg, _ := u.FileToString(outputFile)
                u.PrintlnErr("Failed exporting "+dir+" from VM \""+vm.Name+"\": "+resp.Status()+": "+errorMsg)
                statusCode = 1
            }
        }
    }

    return statusCode
}
